# -*- coding: utf-8 -*-
from OpenSSL import crypto

# Generate public/private key pair
pkey = crypto.PKey( )
pkey.generate_key( crypto.TYPE_RSA , 2048 )

# Create certificate
ca_cert = crypto.X509( )
ca_cert.set_version( 3 )
ca_cert.set_pubkey( pkey )
ca_cert.set_serial_number( 0xdeadbeef )

# Set notBefore/notAfter
ca_cert.set_notBefore( '200812010000+0000' )
ca_cert.set_notAfter( '201212010000+0000' )

# Generate subject
subject = {
	'C'  : 'FR' ,
	'L'  : 'Bordeaux' ,
	'O'  : 'Snake Oil Inc.' ,
	'OU' : 'Sales' ,
	'CN' : 'Root CA' ,
}
cert_name = ca_cert.get_subject( )
for part in ( 'C' , 'L' , 'ST' , 'O' , 'OU' , 'CN' ):
	if part in subject:
		setattr( cert_name , part , subject[ part ] )
ca_cert.set_issuer( crypto.X509Name( cert_name ) )

# Generate X509V3 context
ctx = crypto.X509Context()
ctx.set_subject( ca_cert )
ctx.set_issuer( ca_cert )

# Generate extensions
extensions = []
extensions.append( crypto.X509Extension( 'basicConstraints' , True , 'CA:TRUE,pathlen:0' ) )
extensions.append( crypto.X509Extension( 'subjectKeyIdentifier' , True , 'hash' , context = ctx ) )
ca_cert.add_extensions( extensions )
extensions = [ crypto.X509Extension( 'authorityKeyIdentifier' , False , 'keyid:always,issuer:always' , context = ctx ) ]
ca_cert.add_extensions( extensions )


# Sign the certificate
ca_cert.sign( pkey , 'sha1' )

# Dump certificate
print crypto.dump_certificate( crypto.FILETYPE_PEM , ca_cert )
